@@ -264,6 +264,13 @@ sub _set_from_options {
         $data->{'start'}               = $self->_startstr_to_start($self->_strtok($string, ';'));
     }
 
+    # Timeperiod Transitions
+    elsif($data->{'type'} eq 'TIMEPERIOD TRANSITION') {
+        $data->{'timeperiod'} = $self->_strtok($string, ';');
+        $data->{'from'}       = $self->_strtok($string, ';');
+        $data->{'to'}         = $self->_strtok($string, ';');
+    }
+
     return 1;
 }
 
@@ -323,6 +330,7 @@ sub _statestr_to_state {
     return 2 if $string eq 'UNREACHABLE';
     return 3 if $string eq 'UNKNOWN';
     return 0 if $string eq 'RECOVERY';
+    return 0 if $string eq 'PENDING';
     confess("unknown state: $string");
 }
 
